#!/bin/bash

function check_node_health(){
    rabbitmq_ping=1
    until [[ ${rabbitmq_ping} == 0 ]]; do
        rabbitmq-diagnostics node_health_check > /dev/null 2>&1
        rabbitmq_ping=$?
    done
}

function join_to_existing_cluster(){
    echo "Check if the local rabbit is already part of the cluster ${XRAY_ACTIVE_NODE}"
    rabbitmqctl cluster_status | tr -d '\n' | tr -d ' ' | grep -Po '{nodes,\[.*?\]\}' | grep rabbit@${XRAY_ACTIVE_NODE} 2>/dev/null
    ret_code=$?
    if [[ ! ${ret_code} == 0 ]]; then
        rabbitmqctl stop_app && \
        rabbitmqctl join_cluster rabbit@${XRAY_ACTIVE_NODE} && \
        rabbitmqctl start_app && \
        rabbitmqctl set_policy ha-all "." '{"ha-mode":"all", "ha-sync-mode":"automatic"}' --apply-to all --priority 0
    fi
}

function check_cluster_node_health(){
    node_for_check=${1}
    rabbitmq_ping=1
    max_retry=5
    counter=0
    until [[ ${rabbitmq_ping} == 0 || counter -eq ${max_retry} ]]; do
        ((counter++))
        echo "Checking HEALTH for node ${node_for_check}... (Attempt: ${counter})"
        rabbitmq-diagnostics node_health_check -n ${node_for_check} > /dev/null 2>&1
        rabbitmq_ping=$?
        sleep 1
    done
    if [[ counter -eq ${max_retry} ]]; then
        echo "Reached maximum attempts (${max_retry}) for node ${node_for_check}"
        ask_for_clean_cluster_node ${node_for_check}
    fi
}

function clean_cluster_nodes(){
    nodes=$(rabbitmqctl cluster_status | tr -d '\n' | tr -d ' ' | awk -F ']' '{print $1}' | awk -F'[' '{print $4}' | tr ',' ' ' | tr -d "'")
    echo "Registered nodes are: ${nodes}"
    for node in ${nodes}; do
        check_cluster_node_health ${node}
    done
}

function ask_for_clean_cluster_node(){
    stopped_rabbitmq_node=${1}
    clean_stopped_rabbitmq_node=
    valid_user_choice=false
    until [ ${valid_user_choice} == true ]; do
        read -p "Looks like node '${stopped_rabbitmq_node}' is NOT running. Would you like to remove node '${stopped_rabbitmq_node}' from cluster? [y/n]:" clean_stopped_rabbitmq_node
        userChoiceVerification ${clean_stopped_rabbitmq_node}
    done

    if [[ ${clean_stopped_rabbitmq_node} = [yY] ]]; then
        echo "Removing node '${stopped_rabbitmq_node}' from cluster..."
        rabbitmqctl forget_cluster_node ${stopped_rabbitmq_node}
        if [[ $? -ne 0 ]]; then
            echo "FAILED to remove ${stopped_rabbitmq_node} from cluster"
        fi
    else
        echo "Skipping removal of node ${stopped_rabbitmq_node}..."
        echo "More details:"
        rabbitmq-diagnostics node_health_check -n ${stopped_rabbitmq_node}
    fi
}

function userChoiceVerification() {
    user_choice=${1}
    if [[ ${1} = [nN] ]] || [[ ${user_choice} = [yY] ]]; then
        valid_user_choice=true
    else
        echo "Invalid option: '${user_choice}'. Answer should be 'y' or 'n'."
    fi;
}

check_node_health
join_to_existing_cluster
clean_cluster_nodes